@@ -50,11 +50,14 @@
 
 #include "e-book-backend-kolab.h"
 
+/* This forces the GType to be registered in a way that
+ * avoids a "statement with no effect" compiler warning.
+ * FIXME Use g_type_ensure() once we require GLib 2.34. */
+#define REGISTER_TYPE(type) \
+	(g_type_class_unref (g_type_class_ref (type)))
+
 /*----------------------------------------------------------------------------*/
-/* table of KolabMailAccess objects */
 
-static GHashTable *koma_objects = NULL;
-static GMutex koma_objects_lock;
 static GMutex active_book_views_lock;
 
 /*----------------------------------------------------------------------------*/
@@ -242,29 +245,24 @@ book_backend_kolab_open (EBookBackendSync *backend,
 	CamelKolabIMAPXSettings *kolab_settings = NULL;
 	CamelNetworkSettings *network_settings = NULL;
 	KolabSettingsHandler *ksettings = NULL;
-	KolabMailAccess *tmp_koma = NULL;
 	KolabSyncStrategyID sync_value = KOLAB_SYNC_STRATEGY_DEFAULT;
 	const gchar *extension_name = NULL;
 	const gchar *foldername = NULL;
 	gchar *servername = NULL;
 	gchar *username = NULL;
-	gchar *user_at_server = NULL;
-	gchar *tmp_key = NULL;
 	gboolean online = FALSE;
 	gboolean ok = FALSE;
 	GError *tmp_err = NULL;
 
+	g_return_if_fail (error == NULL || *error == NULL);
 	e_return_data_book_error_if_fail (E_IS_BOOK_BACKEND_KOLAB (backend), E_DATA_BOOK_STATUS_INVALID_ARG);
 	e_return_data_book_error_if_fail (E_IS_DATA_BOOK (book), E_DATA_BOOK_STATUS_INVALID_ARG);
 	/* cancellable may be NULL */
 	(void)only_if_exists; /* FIXME */
-	e_return_data_book_error_if_fail (error == NULL || *error == NULL, E_DATA_BOOK_STATUS_INVALID_ARG);
 
 	self = E_BOOK_BACKEND_KOLAB (backend);
 	priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
 
-	g_mutex_lock (&koma_objects_lock);
-
 	esource = e_backend_get_source (E_BACKEND (backend));
 	if (! E_IS_SOURCE (esource)) {
 		tmp_err = e_data_book_create_error (E_DATA_BOOK_STATUS_OTHER_ERROR,
@@ -284,39 +282,10 @@ book_backend_kolab_open (EBookBackendSync *backend,
 	extension_name = E_SOURCE_EXTENSION_KOLAB_FOLDER;
 	extension = e_source_get_extension (esource, extension_name);
 	sync_value = e_source_kolab_folder_get_sync_strategy (extension);
-
+	
+	online = e_backend_get_online (E_BACKEND (backend));
 	foldername = kolab_util_backend_get_foldername (E_BACKEND (backend));
 
-	/* Check whether we have a KolabMailAccess (KoMA) instance already */
-	user_at_server = g_strdup_printf ("%s@%s",
-	                                  username, servername);
-	ok = g_hash_table_lookup_extended (koma_objects,
-	                                   user_at_server,
-	                                   (gpointer *) &tmp_key,
-	                                   (gpointer *) &tmp_koma);
-
-	if (ok) {
-		/* There already is a KoMA instance for user_at_server.
-		 * Use it and return
-		 */
-		g_object_ref (tmp_koma);
-		priv->book_koma = tmp_koma;
-		ksettings = kolab_mail_access_get_settings_handler (priv->book_koma);
-		kolab_util_backend_prepare_settings (ksettings,
-		                                     NULL,
-		                                     NULL,
-		                                     foldername,
-		                                     &sync_value);
-
-		(void) kolab_util_backend_deploy_mode_by_backend (priv->book_koma,
-		                                                  online,
-		                                                  cancellable,
-		                                                  &tmp_err);
-		goto exit;
-	}
-
-	/* Nope, we need to setup a new KoMA instance and a settings handler */
-
 	/* init subsystems (these are no-ops if already called before) */
 	kolab_util_glib_init ();
 	kolab_util_http_init ();
@@ -349,12 +318,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 
 	/* create new KolabMailAccess instance */
 	priv->book_koma = KOLAB_MAIL_ACCESS (g_object_new (KOLAB_TYPE_MAIL_ACCESS, NULL));
-	g_object_add_toggle_ref (G_OBJECT (priv->book_koma),
-	                         kolab_util_backend_koma_table_cleanup_cb,
-	                         koma_objects);
-	g_hash_table_insert (koma_objects,
-	                     user_at_server,
-	                     priv->book_koma);
 
 	/* configure and bring up KolabMailAccess instance */
 	ok = kolab_mail_access_configure (priv->book_koma,
@@ -370,19 +333,18 @@ book_backend_kolab_open (EBookBackendSync *backend,
 	if (! ok)
 		goto exit;
 
+	(void) kolab_util_backend_deploy_mode_by_backend (priv->book_koma,
+	                                                  online,
+	                                                  cancellable,
+	                                                  &tmp_err);
  exit:
 
 	if (ksettings != NULL)
 		g_object_unref (ksettings);
-
-	g_mutex_unlock (&koma_objects_lock);
-
 	if (servername != NULL)
 		g_free (servername);
 	if (username != NULL)
 		g_free (username);
-	if (user_at_server != NULL)
-		g_free (user_at_server);
 
 	/* do we have an error set? if so, propagate and return */
 	if (tmp_err != NULL) {
@@ -407,6 +369,8 @@ book_backend_kolab_remove (EBookBackendSync *backend,
 {
 	EBookBackendKolab *self = NULL;
 	EBookBackendKolabPrivate *priv = NULL;
+	GError *tmp_err = NULL;
+	gboolean ok = FALSE;
 
 	g_return_if_fail (error == NULL || *error == NULL);
 	e_return_data_book_error_if_fail (E_IS_BOOK_BACKEND_KOLAB (backend), E_DATA_BOOK_STATUS_INVALID_ARG);
@@ -416,9 +380,18 @@ book_backend_kolab_remove (EBookBackendSync *backend,
 	self = E_BOOK_BACKEND_KOLAB (backend);
 	priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
 
-	g_warning ("%s: FIXME implement me", __func__);
-
-	book_backend_kolab_notify_removed (self);
+	if (priv->book_koma != NULL) {
+		ok = kolab_mail_access_shutdown (priv->book_koma,
+		                                 cancellable,
+		                                 &tmp_err);
+		if (ok) {
+			g_object_unref (priv->book_koma);
+			priv->book_koma = NULL;
+			book_backend_kolab_notify_removed (self);
+		} else {
+			g_propagate_error (error, tmp_err);
+		}
+	}
 }
 
 static void
@@ -427,17 +400,18 @@ book_backend_kolab_refresh (EBookBackendSync *backend,
                             GCancellable *cancellable,
                             GError **error)
 {
+#if 0
 	EBookBackendKolab *self = NULL;
 	EBookBackendKolabPrivate *priv = NULL;
-
+#endif
 	g_return_if_fail (error == NULL || *error == NULL);
 	e_return_data_book_error_if_fail (E_IS_BOOK_BACKEND_KOLAB (backend), E_DATA_BOOK_STATUS_INVALID_ARG);
 	e_return_data_book_error_if_fail (E_IS_DATA_BOOK (book), E_DATA_BOOK_STATUS_INVALID_ARG);
 	/* cancellable may be NULL */
-
+#if 0
 	self = E_BOOK_BACKEND_KOLAB (backend);
 	priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
-
+#endif
 	g_warning ("%s: FIXME implement me", __func__);
 }
 
@@ -803,8 +777,8 @@ book_backend_kolab_get_backend_property (EBookBackendSync *backend,
                                          gchar **prop_value,
                                          GError **error)
 {
-	EBookBackendKolab *self = NULL;
-	EBookBackendKolabPrivate *priv = NULL;
+	/* EBookBackendKolab *self = NULL; */
+	/* EBookBackendKolabPrivate *priv = NULL; */
 	gboolean processed = TRUE;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -814,8 +788,8 @@ book_backend_kolab_get_backend_property (EBookBackendSync *backend,
 	e_return_data_book_error_val_if_fail (prop_name != NULL, E_DATA_BOOK_STATUS_INVALID_ARG);
 	e_return_data_book_error_val_if_fail (prop_value != NULL && *prop_value == NULL, E_DATA_BOOK_STATUS_INVALID_ARG);
 
-	self = E_BOOK_BACKEND_KOLAB (backend);
-	priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
+	/* self = E_BOOK_BACKEND_KOLAB (backend); */
+	/* priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self); */
 
 	if (g_str_equal (prop_name, CLIENT_BACKEND_PROPERTY_CAPABILITIES)) {
 		*prop_value = g_strdup ("net,bulk-removes,contact-lists,do-initial-query");
@@ -967,6 +941,7 @@ book_backend_kolab_start_book_view (EBookBackend *backend,
 	GList *econtact_list = NULL;
 	GList *it = NULL;
 	const gchar *query = NULL;
+	const gchar *foldername = NULL;
 	gboolean ok = FALSE;
 	GError *tmp_err = NULL;
 	GError *notify_err = NULL;
@@ -974,14 +949,55 @@ book_backend_kolab_start_book_view (EBookBackend *backend,
 	g_return_if_fail (E_IS_BOOK_BACKEND_KOLAB (backend));
 	g_return_if_fail (E_IS_DATA_BOOK_VIEW (book_view));
 
+	g_mutex_lock (&active_book_views_lock);
+
 	self = E_BOOK_BACKEND_KOLAB (backend);
 	priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
 
-	g_mutex_lock (&active_book_views_lock);
-
 	g_object_ref (book_view); /* unref()d in stop_view() */
 
 	query = e_data_book_view_get_card_query (book_view);
+	foldername = kolab_util_backend_get_foldername (E_BACKEND (backend));
+
+	/* try to switch into online mode. if that fails,
+	 * we assume there's no network or service available
+	 * and continue in offline mode.
+	 *
+	 * FIXME we need some error reporting here. Going
+	 * online may block for any kind of reasons, in which
+	 * case the view will never get started...
+	 */
+	ok = kolab_util_backend_deploy_mode_by_backend (priv->book_koma,
+	                                                TRUE, /* go online */
+	                                                NULL, /* FIXME: GCancellable */
+	                                                &tmp_err);
+	if (! ok) {
+		g_warning ("%s()[%u]: %s", __func__, __LINE__, tmp_err->message);
+		g_error_free (tmp_err);
+		tmp_err = NULL;
+	}
+
+	/* Up to 3.4, we've been synchronizing folder metadata and
+	 * PIM payload data with the Kolab server when going online.
+	 * This can now be done in a separate synchronization function, which
+	 * can be called upon per user request or at any place internally
+	 * where it seems fit (e.g. when switching views - but beware,
+	 * that operation can become quite bulky and time consuming).
+	 * Until we get a synchronize() backend API function, we will
+	 * put a sync point here to mimick the previous behaviour.
+	 *
+	 * We just pretend that going online went okay.
+	 */
+	ok = kolab_mail_access_synchronize (priv->book_koma,
+	                                    foldername,
+	                                    TRUE, /* full sync */
+	                                    NULL, /* FIXME: GCancellable */
+	                                    &tmp_err);
+	if (! ok) {
+		g_warning ("%s()[%u]: %s", __func__, __LINE__, tmp_err->message);
+		g_error_free (tmp_err);
+		tmp_err = NULL;
+	}
 
 	/* First update the BackendCache to the current situation. */
 	ok = kolab_util_contact_cache_update_on_query (E_BOOK_BACKEND_SYNC (backend),
@@ -1016,30 +1032,22 @@ book_backend_kolab_start_book_view (EBookBackend *backend,
 
 	e_data_book_view_notify_complete (book_view, notify_err);
 
-	g_mutex_unlock (&active_book_views_lock);
-
 	if (notify_err != NULL)
 		g_error_free (notify_err);
 
 	if (econtact_list != NULL)
 		g_list_free (econtact_list);
 
+	g_mutex_unlock (&active_book_views_lock);
 }
 
 static void
 book_backend_kolab_stop_book_view (EBookBackend *backend,
                                    EDataBookView *book_view)
 {
-	EBookBackendKolab *self = NULL;
-	EBookBackendKolabPrivate *priv = NULL;
-	EDataBookView *bv = NULL;
-
 	g_return_if_fail (E_IS_BOOK_BACKEND_KOLAB (backend));
 	g_return_if_fail (E_IS_DATA_BOOK_VIEW (book_view));
 
-	self = E_BOOK_BACKEND_KOLAB (backend);
-	priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
-
 	g_mutex_lock (&active_book_views_lock);
 
 	e_data_book_view_notify_complete (book_view, NULL);
@@ -1057,14 +1065,6 @@ e_book_backend_kolab_init (EBookBackendKolab *backend)
 {
 	EBookBackendKolabPrivate *priv = E_BOOK_BACKEND_KOLAB_PRIVATE (backend);
 
-	g_mutex_lock (&koma_objects_lock);
-	if (koma_objects == NULL)
-		koma_objects = g_hash_table_new_full (g_direct_hash,
-		                                      g_direct_equal,
-		                                      g_free,
-		                                      g_object_unref);
-	g_mutex_unlock (&koma_objects_lock);
-
 	g_debug ("%s()[%u] called.", __func__, __LINE__);
 
 	priv->book_koma = NULL;
@@ -1081,22 +1081,12 @@ e_book_backend_kolab_dispose (GObject *object)
 	EBookBackendKolabPrivate *priv = E_BOOK_BACKEND_KOLAB_PRIVATE (self);
 
 	g_debug ("%s()[%u] called.", __func__, __LINE__);
-#if 0 /* FIXME */
-	if (e_book_backend_is_loaded (self)) {
-		e_book_backend_set_is_writable (self, FALSE);
-		e_book_backend_notify_writable (self, FALSE);
-		e_book_backend_notify_connection_status (self, FALSE);
-		e_book_backend_set_is_loaded (self, FALSE);
-	}
-#endif
-
-	g_warning ("%s()[%u] FIXME KolabMailAccess (table) disposal", __func__, __LINE__);
-#if 0 /* FIXME (check g_object_toggle_ref cleanup callback) */
 	if (priv->book_koma != NULL) {
 		g_object_unref (priv->book_koma);
 		priv->book_koma = NULL;
 	}
-#endif
+
+	G_OBJECT_CLASS (e_book_backend_kolab_parent_class)->dispose (object);
 }
 
 static void
@@ -1144,6 +1134,10 @@ e_book_backend_kolab_class_init (EBookBackendKolabClass *klass)
 	sync_class->get_contact_sync = book_backend_kolab_get_contact;
 	sync_class->get_contact_list_sync = book_backend_kolab_get_contact_list;
 	sync_class->get_contact_list_uids_sync = book_backend_kolab_get_contact_list_uids;
+
+	/* Register relevant ESourceExtension types. */
+	REGISTER_TYPE (E_TYPE_SOURCE_KOLAB_FOLDER);
+
 } /* e_book_backend_kolab_class_init () */
 
 
