@@ -52,11 +52,14 @@
 #include "kolab-util-calendar-cache.h"
 #include "e-cal-backend-kolab.h"
 
+/* This forces the GType to be registered in a way that
+ * avoids a "statement with no effect" compiler warning.
+ * FIXME Use g_type_ensure() once we require GLib 2.34. */
+#define REGISTER_TYPE(type) \
+	(g_type_class_unref (g_type_class_ref (type)))
+
 /*----------------------------------------------------------------------------*/
-/* table of KolabMailAccess objects */
 
-static GHashTable *koma_objects = NULL;
-static GMutex koma_objects_lock;
 static GMutex active_cal_views_lock;
 
 /*----------------------------------------------------------------------------*/
@@ -266,7 +269,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	CamelKolabIMAPXSettings *kolab_settings = NULL;
 	CamelNetworkSettings *network_settings = NULL;
 	KolabSettingsHandler *ksettings = NULL;
-	KolabMailAccess *tmp_koma = NULL;
 	KolabSyncStrategyID sync_value = KOLAB_SYNC_STRATEGY_DEFAULT;
 	icalcomponent_kind icalkind = ICAL_VEVENT_COMPONENT;
 	const gchar *extension_name = NULL;
@@ -275,8 +277,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	gchar *cache_filename = NULL;
 	gchar *servername = NULL;
 	gchar *username = NULL;
-	gchar *user_at_server = NULL;
-	gchar *tmp_key = NULL;
 	gboolean online = FALSE;
 	gboolean ok = FALSE;
 	GError *tmp_err = NULL;
@@ -290,8 +290,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	self = E_CAL_BACKEND_KOLAB (backend);
 	priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
-	g_mutex_lock (&koma_objects_lock);
-
 	icalkind = e_cal_backend_get_kind (E_CAL_BACKEND (backend));
 	/* TODO: this has to be set according to the get_kind() method */
 	switch (icalkind) {
@@ -327,6 +325,12 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	g_debug ("%s()[%u] servername = %s", __func__, __LINE__, servername);
 	g_debug ("%s()[%u]   username = %s", __func__, __LINE__, username);
 
+	/* TODO this is sort of hackish, we cannot be sure that
+	 *      'user@server' is truly the Kolab email address
+	 *      of the Kolab user
+	 */
+	priv->user_email = g_strdup_printf ("%s@%s", username, servername);
+
 	/* Initialize backend cache */
 	if (priv->cal_cache != NULL) {
 		g_object_unref (priv->cal_cache);
@@ -345,50 +349,9 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	extension = e_source_get_extension (esource, extension_name);
 	sync_value = e_source_kolab_folder_get_sync_strategy (extension);
 
+	online = e_backend_get_online (E_BACKEND (backend));
 	foldername = kolab_util_backend_get_foldername (E_BACKEND (backend));
 
-	/* Check whether we have a KolabMailAccess (KoMA) instance already */
-	user_at_server = g_strdup_printf ("%s@%s",
-	                                  username, servername);
-	ok = g_hash_table_lookup_extended (koma_objects,
-	                                   user_at_server,
-	                                   (gpointer *) &tmp_key,
-	                                   (gpointer *) &tmp_koma);
-
-	if (ok) {
-		/* There already is a KoMA instance for user_at_server.
-		 * Use it and return
-		 */
-		g_object_ref (tmp_koma);
-		priv->cal_koma = tmp_koma;
-		ksettings = kolab_mail_access_get_settings_handler (priv->cal_koma);
-		kolab_util_backend_prepare_settings (ksettings,
-		                                     NULL,
-		                                     NULL,
-		                                     foldername,
-		                                     &sync_value);
-
-		(void) kolab_util_backend_deploy_mode_by_backend (priv->cal_koma,
-		                                                  online,
-		                                                  cancellable,
-		                                                  &tmp_err);
-		goto exit;
-	}
-
-	/* Nope, we need to setup a new KoMA instance and a settings handler */
-
-	/* init subsystems (these are no-ops if already called before) */
-	kolab_util_glib_init ();
-	kolab_util_http_init ();
-	/* libcamel
-	 * Curl init may configure the underlying SSL lib,
-	 * but as far as SSL goes, we want Camel to rule here
-	 * TODO check whether Camel session needs to be initialized before or after libcurl.
-	 */
-	ok = kolab_util_camel_init (&tmp_err);
-	if (! ok)
-		goto exit;
-
 	/* Configure settings handler */
 	ksettings = kolab_settings_handler_new (kolab_settings);
 	ok = kolab_settings_handler_configure (ksettings,
@@ -409,12 +372,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 
 	/* create new KolabMailAccess instance */
 	priv->cal_koma = KOLAB_MAIL_ACCESS (g_object_new (KOLAB_TYPE_MAIL_ACCESS, NULL));
-	g_object_add_toggle_ref (G_OBJECT (priv->cal_koma),
-	                         kolab_util_backend_koma_table_cleanup_cb,
-	                         koma_objects);
-	g_hash_table_insert (koma_objects,
-	                     user_at_server,
-	                     priv->cal_koma);
 
 	/* configure and bring up KolabMailAccess instance */
 	ok = kolab_mail_access_configure (priv->cal_koma,
@@ -430,25 +387,18 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	if (! ok)
 		goto exit;
 
-	/* TODO this is sort of hackish, we cannot be sure that
-	 *      'user@server' is truly the Kolab email address
-	 *      of the Kolab user
-	 */
-	priv->user_email = g_strdup (user_at_server);
-
+	(void) kolab_util_backend_deploy_mode_by_backend (priv->cal_koma,
+	                                                  online,
+	                                                  cancellable,
+	                                                  &tmp_err);
  exit:
 
 	if (ksettings != NULL)
 		g_object_unref (ksettings);
-
-	g_mutex_unlock (&koma_objects_lock);
-
 	if (servername != NULL)
 		g_free (servername);
 	if (username != NULL)
 		g_free (username);
-	if (user_at_server != NULL)
-		g_free (user_at_server);
 
 	/* do we have an error set? if so, propagate and return */
 	if (tmp_err != NULL) {
@@ -472,33 +422,39 @@ cal_backend_kolab_remove (ECalBackendSync *backend,
                           GError **error)
 {
 	ECalBackendKolab *self = NULL;
-	/* ECalBackendKolabPrivate *priv = NULL; */
+	ECalBackendKolabPrivate *priv = NULL;
+	GError *tmp_err = NULL;
+	gboolean ok = FALSE;
 
 	g_return_if_fail (error == NULL || *error == NULL);
 	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_KOLAB (backend), InvalidArg);
+	/* cancellable may be NULL */
 	e_return_data_cal_error_if_fail (E_IS_DATA_CAL (cal), InvalidArg);
-	(void)cancellable; /* FIXME */ /* cancellable may be NULL */
 
 	self = E_CAL_BACKEND_KOLAB (backend);
-	/* priv = E_CAL_BACKEND_KOLAB_PRIVATE (self); */
+	priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
-	g_warning ("%s: FIXME implement me", __func__);
-
-#if 0 /* FIXME old */
 	if (priv->cal_koma != NULL) {
-		g_object_unref (priv->cal_koma);
-		priv->cal_koma = NULL;
+		ok = kolab_mail_access_shutdown (priv->cal_koma,
+		                                 cancellable,
+		                                 &tmp_err);
+		if (ok) {
+			g_object_unref (priv->cal_koma);
+			priv->cal_koma = NULL;
+		} else {
+			g_propagate_error (error, tmp_err);
+		}
 	}
 
-	/* Remove backend cache */
 	if (priv->cal_cache != NULL) {
 		(void) e_file_cache_remove (E_FILE_CACHE (priv->cal_cache));
 		g_object_unref (priv->cal_cache);
 		priv->cal_cache = NULL;
 	}
-#endif
 
-	cal_backend_kolab_notify_removed (self);
+	if (ok) {
+		cal_backend_kolab_notify_removed (self);
+	}
 }
 
 static void
@@ -854,6 +810,7 @@ cal_backend_kolab_create_objects (ECalBackendSync *backend,
 	ECalComponent *ecalcomp = NULL;
 	ECalComponent *tzcomp = NULL;
 	const gchar *calobj = NULL;
+	const gchar *uid = NULL;
 	GError *tmp_err = NULL;
 	gboolean ok = FALSE;
 
@@ -865,7 +822,7 @@ cal_backend_kolab_create_objects (ECalBackendSync *backend,
 	e_return_data_cal_error_if_fail (uids != NULL && *uids == NULL, InvalidArg);
 	e_return_data_cal_error_if_fail (new_components != NULL && *new_components == NULL, InvalidArg);
 
-	if (calobjs->next) {
+	if (g_slist_next (calobjs) != NULL) {
 		g_propagate_error (error, e_data_cal_create_error (UnsupportedMethod, _("Kolab does not support bulk additions")));
 		return;
 	}
@@ -903,7 +860,7 @@ cal_backend_kolab_create_objects (ECalBackendSync *backend,
 		if (tzid != NULL) {
 			/* TZID is set, but no timezone is available. Currently
 			 * this may happen, if an object is copied to an yet unused/empty
-			 * calendar. We do not want such crap in the kolab database.
+			 * calendar.
 			 */
 			g_free (tzid);
 			g_object_unref (ecalcomp);
@@ -946,7 +903,6 @@ cal_backend_kolab_create_objects (ECalBackendSync *backend,
 		return;
 	}
 
-	g_object_unref (ecalcomp);
 	if (tzcomp != NULL)
 		g_object_unref (tzcomp);
 
@@ -963,6 +919,9 @@ cal_backend_kolab_create_objects (ECalBackendSync *backend,
 	 *      a copy in *new_component (and set *uid) before returning
 	 *      from this function
 	 */
+	e_cal_component_get_uid (ecalcomp, &uid);
+	*uids = g_slist_prepend (*uids, g_strdup (uid));
+	*new_components = g_slist_prepend (*new_components, (gpointer) ecalcomp);
 }
 
 /**
@@ -1347,7 +1306,7 @@ cal_backend_kolab_remove_objects (ECalBackendSync *backend,
 
 	if (*old_components != NULL)
 		oc_data = (*old_components)->data;
-	if (*new_components = NULL)
+	if (*new_components != NULL)
 		nc_data = (*new_components)->data;
 	
 	e_cal_backend_notify_component_removed (E_CAL_BACKEND (backend),
@@ -1742,7 +1701,7 @@ cal_backend_kolab_start_view (ECalBackend *backend,
 	GList *it = NULL;
 	GSList *iCal_objects = NULL;
 	const gchar *query = NULL;
-	const gchar *foldername;
+	const gchar *foldername = NULL;
 	GError *tmp_err = NULL;
 	GError *view_err = NULL;
 	gboolean ok = FALSE;
@@ -1750,17 +1709,56 @@ cal_backend_kolab_start_view (ECalBackend *backend,
 	g_return_if_fail (E_IS_CAL_BACKEND_KOLAB (backend));
 	g_return_if_fail (E_IS_DATA_CAL_VIEW (view));
 
+	g_mutex_lock (&active_cal_views_lock);
+
 	self = E_CAL_BACKEND_KOLAB (backend);
 	priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
 
-	g_mutex_lock (&active_cal_views_lock);
-
 	g_object_ref (view); /* unref()d in stop_view() */
 
 	query = e_data_cal_view_get_text (view);
-
 	foldername = kolab_util_backend_get_foldername (E_BACKEND (backend));
 
+	/* try to switch into online mode. if that fails,
+	 * we assume there's no network or service available
+	 * and continue in offline mode.
+	 *
+	 * FIXME we need some error reporting here. Going
+	 * online may block for any kind of reasons, in which
+	 * case the view will never get started...
+	 */
+	ok = kolab_util_backend_deploy_mode_by_backend (priv->cal_koma,
+	                                                TRUE, /* go online */
+	                                                NULL, /* FIXME: GCancellable */
+	                                                &tmp_err);
+	if (! ok) {
+		g_warning ("%s()[%u]: %s", __func__, __LINE__, tmp_err->message);
+		g_error_free (tmp_err);
+		tmp_err = NULL;
+	}
+
+	/* Up to 3.4, we've been synchronizing folder metadata and
+	 * PIM payload data with the Kolab server when going online.
+	 * This can now be done in a separate synchronization function, which
+	 * can be called upon per user request or at any place internally
+	 * where it seems fit (e.g. when switching views - but beware,
+	 * that operation can become quite bulky and time consuming).
+	 * Until we get a synchronize() backend API function, we will
+	 * put a sync point here to mimick the previous behaviour.
+	 *
+	 * We just pretend that going online went okay.
+	 */
+	ok = kolab_mail_access_synchronize (priv->cal_koma,
+	                                    foldername,
+	                                    TRUE, /* full sync */
+	                                    NULL, /* FIXME: GCancellable */
+	                                    &tmp_err);
+	if (! ok) {
+		g_warning ("%s()[%u]: %s", __func__, __LINE__, tmp_err->message);
+		g_error_free (tmp_err);
+		tmp_err = NULL;
+	}
+
 	ok = kolab_util_calendar_cache_update_on_query (E_CAL_BACKEND_SYNC (backend),
 	                                                priv->cal_cache,
 	                                                priv->cal_koma,
@@ -1824,8 +1822,6 @@ cal_backend_kolab_start_view (ECalBackend *backend,
 
 	e_data_cal_view_notify_complete (view, view_err);
 
-	g_mutex_unlock (&active_cal_views_lock);
-
 	if (view_err != NULL)
 		g_error_free (view_err);
 
@@ -1836,6 +1832,7 @@ cal_backend_kolab_start_view (ECalBackend *backend,
 	if (uid_list != NULL)
 		kolab_util_glib_glist_free (uid_list);
 
+	g_mutex_unlock (&active_cal_views_lock);
 }
 
 static void
@@ -1868,14 +1865,7 @@ e_cal_backend_kolab_init (ECalBackendKolab *backend)
 {
 	ECalBackendKolab *self = E_CAL_BACKEND_KOLAB (backend);
 	ECalBackendKolabPrivate *priv = E_CAL_BACKEND_KOLAB_PRIVATE (self);
-
-	g_mutex_lock (&koma_objects_lock);
-	if (koma_objects == NULL)
-		koma_objects = g_hash_table_new_full (g_direct_hash,
-		                                      g_direct_equal,
-		                                      g_free,
-		                                      g_object_unref);
-	g_mutex_unlock (&koma_objects_lock);
+	GError *tmp_err = NULL;
 
 	g_debug ("%s()[%u] called.", __func__, __LINE__);
 
@@ -1887,6 +1877,20 @@ e_cal_backend_kolab_init (ECalBackendKolab *backend)
 	priv->source_type = 0;
 	priv->mode_switch_err = NULL;
 
+	/* init subsystems (these are no-ops if already called before) */
+	kolab_util_glib_init ();
+	kolab_util_folder_init ();
+	kolab_util_http_init ();
+	/* libcamel
+	 * Curl init may configure the underlying SSL lib,
+	 * but as far as SSL goes, we want Camel to rule here
+	 * TODO check whether Camel session needs to be initialized before or after libcurl.
+	 */
+	if (! kolab_util_camel_init (&tmp_err)) {
+		g_error ("%s()[%u]: %s", __func__, __LINE__, tmp_err->message);
+		return;
+	}
+
 	g_signal_connect (E_BACKEND (backend), "notify::online", G_CALLBACK (cal_backend_kolab_signal_online_cb), NULL);
 } /* e_cal_backend_kolab_init () */
 
@@ -1898,13 +1902,10 @@ e_cal_backend_kolab_dispose (GObject *object)
 
 	g_debug ("%s()[%u] called.", __func__, __LINE__);
 
-	g_warning ("%s()[%u] FIXME KolabMailAccess (table) disposal", __func__, __LINE__);
-#if 0 /* FIXME (check g_object_toggle_ref cleanup callback) */
 	if (priv->cal_koma != NULL) {
 		g_object_unref (priv->cal_koma);
 		priv->cal_koma = NULL;
 	}
-#endif
 	if (priv->default_zone != NULL) {
 		g_object_unref (priv->default_zone);
 		priv->default_zone = NULL;
@@ -1977,6 +1978,9 @@ e_cal_backend_kolab_class_init (ECalBackendKolabClass *klass)
 	sync_class->get_timezone_sync = cal_backend_kolab_get_timezone;
 	sync_class->add_timezone_sync = cal_backend_kolab_add_timezone;
 
+	/* Register relevant ESourceExtension types. */
+	REGISTER_TYPE (E_TYPE_SOURCE_KOLAB_FOLDER);
+
 } /* e_cal_backend_kolab_class_init () */
 
 
