@@ -63,6 +63,7 @@ imapx_extd_server_register_desc_list (CamelIMAPXServer *is,
 static gpointer
 imapx_extd_server_register_untagged_fn (gpointer data)
 {
+	static GMutex reg_lock;
 	CamelIMAPXServer *is = CAMEL_IMAPX_SERVER (data);
 	const CamelIMAPXUntaggedRespHandlerDesc *prev = NULL;
 	KolabGConstList *desc_lst = NULL;
@@ -75,6 +76,8 @@ imapx_extd_server_register_untagged_fn (gpointer data)
 	 * are registered here.
 	 */
 
+	g_mutex_lock (&reg_lock);
+	
 	/* METADATA handlers */
 	desc_lst = camel_imapx_extd_server_metadata_get_handler_descriptors ();
 	prev = imapx_extd_server_register_desc_list (is, desc_lst);
@@ -88,6 +91,7 @@ imapx_extd_server_register_untagged_fn (gpointer data)
 	kolab_util_glib_gconstlist_free (desc_lst);
 #endif
  exit:
+	g_mutex_unlock (&reg_lock);
 	return (gpointer) prev;
 }
 
@@ -98,24 +102,36 @@ camel_imapx_extd_server_init (CamelIMAPXServer *is,
                               GCancellable *cancellable,
                               GError **err)
 {
-	static GOnce my_once = G_ONCE_INIT;
-
+	gpointer prev = NULL;
+	static GMutex init_lock;
+	static GHashTable *inited_tbl = NULL;
+	
 	g_assert (CAMEL_IS_IMAPX_SERVER (is));
 	(void)cancellable; /* cancellable may be NULL */
 	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
 
-	g_once (&my_once,
-	        imapx_extd_server_register_untagged_fn,
-	        is);
+	g_mutex_lock (&init_lock);
 
-	if (my_once.retval != NULL) {
-		g_set_error (err,
-		             CAMEL_IMAPX_ERROR,
-		             1, /* define and set a sensible code here */
-		             _("A CamelIMAPXServer extended untagged response handler has been registered twice"));
-		return FALSE;
+	if (inited_tbl == NULL) {
+		inited_tbl = g_hash_table_new_full (g_direct_hash,
+		                                    g_direct_equal,
+		                                    NULL,
+		                                    NULL);
 	}
 
+	if (g_hash_table_lookup (inited_tbl, (gpointer) is) == NULL) {	
+		prev = imapx_extd_server_register_untagged_fn (is);
+		g_hash_table_insert (inited_tbl, (gpointer) is, (gpointer) is);
+		if (prev != NULL) {
+			g_warning ("%s()[%u] %s",
+			           __func__,
+			           __LINE__,		           
+			           _("A CamelIMAPXServer extended untagged response handler has been registered twice"));
+		}
+	}
+
+	g_mutex_unlock (&init_lock);
+
 	return TRUE;
 }
 
