@@ -1391,6 +1391,33 @@ kolab_mail_imap_client_go_offline (KolabMailImapClient *self,
 	return TRUE;
 }
 
+/*----------------------------------------------------------------------------*/
+/* password handling */
+
+const gchar*
+kolab_mail_imap_client_get_password (KolabMailImapClient *self)
+{
+	KolabMailImapClientPrivate *priv = NULL;
+	const gchar *passwd = NULL;
+	
+	g_assert (KOLAB_IS_MAIL_IMAP_CLIENT (self));
+
+	priv = KOLAB_MAIL_IMAP_CLIENT_PRIVATE (self);
+
+	/* If we successfully authenticated before, our CamelService
+	 * will know the password which was used for authentication.
+	 * When starting in offline mode, no password may be known,
+	 * so we may return NULL here. We need the password outside
+	 * Camel for the (X)FB triggers only, which are not sent unless
+	 * we went online, so no trouble to be expected here.
+	 */
+	
+	g_return_val_if_fail (priv->store != NULL, NULL);
+	passwd = camel_service_get_password (CAMEL_SERVICE (priv->store));
+
+	return passwd;
+}
+
 /*----------------------------------------------------------------------------*/
 /* folders */
 
@@ -1424,7 +1451,7 @@ kolab_mail_imap_client_get_folder_type (KolabMailImapClient *self,
 	GError *tmp_err = NULL;
 
 	g_assert (KOLAB_IS_MAIL_IMAP_CLIENT (self));
-	g_assert (foldername != NULL);
+	g_return_val_if_fail (foldername != NULL, KOLAB_FOLDER_TYPE_INVAL);
 	/* cancellable may be NULL */
 	g_return_val_if_fail (err == NULL || *err == NULL, KOLAB_FOLDER_TYPE_INVAL);
 
@@ -1496,24 +1523,23 @@ kolab_mail_imap_client_query_folder_info_online (KolabMailImapClient *self,
 		                                                  cancellable,
 		                                                  &tmp_err);
 		if (tmp_err != NULL) {
-			g_debug ("%s()[%i]: %s", __func__, __LINE__, tmp_err->message);
+			g_warning ("%s()[%i]: %s", __func__, __LINE__, tmp_err->message);
 			g_error_free (tmp_err);
 			tmp_err = NULL;
+		} else {
+			desc = kolab_util_folder_descriptor_new (foldername,
+			                                         type_id);
+			if (desc != NULL)
+				folder_desc = g_list_prepend (folder_desc, desc);
+			else
+				g_warning ("%s()[%u]: NULL folder descriptor for folder '%s', type '%u'",
+				           __func__, __LINE__, foldername, type_id);
 		}
-
-		desc = kolab_util_folder_descriptor_new (foldername,
-		                                         type_id);
-		folder_desc = g_list_prepend (folder_desc, desc);
-
+		
 		folder_names_ptr = g_list_next (folder_names_ptr);
 	}
 
-	if (tmp_err != NULL) {
-		g_propagate_error (err, tmp_err);
-		kolab_util_glib_glist_free (folder_names);
-		kolab_util_folder_descriptor_glist_free (folder_desc);
-		return NULL;
-	}
+	kolab_util_glib_glist_free (folder_names);
 
 	return folder_desc;
 }
