@@ -1151,6 +1151,13 @@ mail_synchronizer_uids_update_infodb (KolabMailSynchronizer *self,
 		g_propagate_error (err, tmp_err);
 		return FALSE;
 	}
+	if (folders_lst == NULL) {
+		g_set_error (err,
+		             KOLAB_BACKEND_ERROR,
+		             KOLAB_BACKEND_ERROR_INFODB_NOFOLDER,
+		             _("No folder information in internal database"));
+		return FALSE;
+	}
 
 	if (foldername != NULL) {
 		GHashTable *folders_tbl = NULL;
