@@ -39,7 +39,8 @@ i_evo_store_add_prop(I_common *i_common, icalproperty *prop, icalcomponent_kind
 		comp = icalcomponent_new(kind);
 	prop = icalproperty_new_clone(prop);
 	icalcomponent_add_property(comp, prop);
-	i_common->evolution_store = icalcomponent_as_ical_string(comp);
+	/* memory returned by *_as_ical_string() is owned by libical */
+	i_common->evolution_store = g_strdup (icalcomponent_as_ical_string(comp));
 	icalcomponent_free(comp);
 }
 
