@@ -56,7 +56,6 @@
 #include <glib/gi18n-lib.h>
 
 /* HTTP lib specifics */
-#include <libsoup/soup-gnome.h>
 #include <curl/curl.h>
 
 #include "camel-kolab-stream.h"
@@ -84,16 +83,6 @@ kolab_util_http_error_quark (void)
 	return g_quark_from_static_string ("kolab-util-http-error-quark");
 }
 
-/* TODO change this to conditional compilation once
- *      things have settled.
- *      We keep this for now to make sure all variants
- *      compile cleanly
- */
-typedef enum {
-	KOLAB_UTIL_HTTP_LIB_SOUP = 0,
-	KOLAB_UTIL_HTTP_LIB_CURL
-} KolabUtilHttpLib;
-
 /*----------------------------------------------------------------------------*/
 
 /* TODO get rid of these globals if possible */
@@ -133,99 +122,6 @@ kolab_util_http_job_free (KolabUtilHttpJob *job)
 	g_free (job);
 }
 
-/*----------------------------------------------------------------------------*/
-/* LibSoup */
-#if 0
-static void
-kolab_util_http_soup_authenticate (SoupSession *session,
-                                   SoupMessage *msg,
-                                   SoupAuth *auth,
-                                   gboolean retrying,
-                                   gpointer data)
-{
-	KolabUtilHttpJob *job;
-
-	(void)session;
-	(void)msg;
-
-	/* no second auth attempt */
-	if (retrying)
-		return;
-
-	job = (KolabUtilHttpJob*) data;
-	g_assert( job != NULL);
-	soup_auth_authenticate (auth,
-	                        job->url->user,
-	                        job->url->passwd);
-}
-
-static gssize
-kolab_util_http_soup_get (const KolabUtilHttpJob *job, GError **error)
-{
-	g_error ("%s: not implemented.", __func__);
-	SoupMessage *msg = NULL;
-	CamelStream *mem = NULL;
-	guint status = 0;
-	gssize nbytes = -1;
-	gchar *url_string = NULL;
-
-	static gulong kolab_soup_auth_handler_id = 0; /* TODO is 0 an invalid id? */
-
-	g_return_val_if_fail (error == NULL || *error == NULL, -1);
-
-	/* connect the "authenticate" signal handler,
-	 * disconnect a previously connected one.
-	 * TODO: check whether we need to disconnect-connect
-	 *       a handler here (KolabUtilHttpJob* may point to
-	 *       an address different from the one we were
-	 *       called upon last time)
-	 */
-	if (g_signal_handler_is_connected (kolab_soup_session,
-	                                   kolab_soup_auth_handler_id))
-		g_signal_handler_disconnect (kolab_soup_session,
-		                             kolab_soup_auth_handler_id);
-	kolab_soup_auth_handler_id = g_signal_connect (kolab_soup_session,
-	                                               "authenticate",
-	                                               G_CALLBACK (kolab_util_http_soup_authenticate),
-	                                               (gpointer)job);
-
-	/* issue HTTP GET */
-	url_string = camel_url_to_string (job->url, CAMEL_URL_HIDE_PASSWORD | CAMEL_URL_HIDE_AUTH);
-	msg = soup_message_new (SOUP_METHOD_GET, url_string);
-	g_free (url_string);
-	/* soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT); */
-	status = soup_session_send_message (kolab_soup_session, msg);
-
-	/* copy msg contents into buffer */
-	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		mem = camel_kolab_stream_new_memstream (job->buffer);
-		nbytes = camel_stream_write (mem,
-		                             (gchar*)msg->response_body->data,
-		                             (gsize)msg->response_body->length);
-		camel_kolab_stream_free (mem);
-		g_debug ("%s: %d bytes received", __func__, nbytes);
-	}
-	else {
-		KolabUtilHttpError error_code;
-		if (status > 500)
-			error_code = KOLAB_UTIL_HTTP_ERROR_SERVER_ERROR;
-		else if (status > 400)
-			error_code = KOLAB_UTIL_HTTP_ERROR_CLIENT_ERROR;
-		else /* if (status > 100) */
-			/* 1xx informational and 2xx is considered success, then why are we here? */
-			error_code = KOLAB_UTIL_HTTP_ERROR_UNKNOWN_ERROR;
-
-		g_set_error (error,
-		             KOLAB_UTIL_HTTP_ERROR,
-		             error_code,
-		             _("Libsoup failed with status %u"),
-		             status);
-	}
-
-	return nbytes;
-}
-#endif
-
 /*----------------------------------------------------------------------------*/
 /* LibCurl */
 
@@ -441,11 +337,6 @@ kolab_util_http_init (void)
 
 	if (http_is_initialized == TRUE)
 		return;
-#if 0
-	/* libsoup */
-	kolab_soup_session = (SoupSession*)soup_session_sync_new ();
-	g_assert (kolab_soup_session != NULL);
-#endif
 
 	/* libcurl */
 
@@ -483,11 +374,6 @@ kolab_util_http_shutdown (void)
 	/* libcurl */
 	curl_easy_cleanup (kolab_curl_handle);
 	curl_global_cleanup ();
-
-#if 0
-	/* libsoup */
-	g_object_unref (kolab_soup_session);
-#endif
 }
 
 /*----------------------------------------------------------------------------*/
@@ -505,74 +391,9 @@ kolab_util_http_shutdown (void)
 gssize
 kolab_util_http_get (KolabUtilHttpJob *job, GError **error)
 {
-	gssize nbytes = -1;
-
-	nbytes = util_http_curl_get (job, error);
-
-#if 0
-	/* TODO change this to conditional compilation once
-	 *      things have settled.
-	 *      We keep this for now to make sure all variants
-	 *      compile cleanly
-	 */
-	KolabUtilHttpLib httplib = KOLAB_UTIL_HTTP_LIB_CURL;
-
-	switch (httplib) {
-	case KOLAB_UTIL_HTTP_LIB_SOUP:
-		nbytes = kolab_util_http_soup_get (job, error);
-		break;
-	case KOLAB_UTIL_HTTP_LIB_CURL:
-		nbytes = kolab_util_http_curl_get (job, error);
-		break;
-	default:
-		/* can't happen */
-		g_assert_not_reached();
-	}
-#endif
-
-	return nbytes;
+	return util_http_curl_get (job, error);
 }
 
-#if 0
-/**
- * kolab_util_http_new_hostname_from_url:
- * @url_string: the URL to extract the hostname part from
- *
- * The returned string should be freed using g_free() if no longer needed.
- *
- * Returns: The host name part from a given URL
- */
-gchar*
-kolab_util_http_new_hostname_from_url (const gchar *url_string)
-{
-	/* TODO move this to http-independent place
-	 *      and rename
-	 */
-	CamelURL *url = NULL;
-	gchar *hostname = NULL;
-	CamelException *ex = NULL;
-
-	g_assert (url_string != NULL);
-
-	ex = camel_exception_new ();
-	url = camel_url_new (url_string, ex);
-	if (camel_exception_is_set (ex)) {
-		g_warning ("%s: %s",
-		           __func__, camel_exception_get_description (ex));
-		if (url != NULL)
-			camel_url_free (url);
-		url = NULL;
-	}
-	camel_exception_free (ex);
-	if (url == NULL)
-		return NULL;
-
-	hostname = g_strdup (url->host);
-	camel_url_free (url);
-	return hostname;
-}
-#endif
-
 /**
  * kolab_util_http_protocol_is_ssl:
  * @url_string: the HTTP URL to check for SSL use. Must be
